untrusted comment: verify with signify key from research.exoticsilicon.com
RWRn5d3Yx35u0zXkW5+w2hn2JutEjWE5GwvSopqj4MgFuhHDbMtAFhj3Bu28HcsxzVs7S4n6IaeDDE4lYWqtTsG7hEH4fApkcAs=
This patchset is intended to be applied to OpenBSD 7.2-release:

# cd /usr/src/
# patch < screendump_kernel_patches

For more information about this patchset, and to download the accompanying userland utilities, please visit:

https://research.exoticsilicon.com/articles/console_screendumps

THIS PATCH IS PROVIDED 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
EXOTIC SILICON BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS PATCH, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--- sys/dev/wscons/wsconsio.h	Fri Jul 15 14:57:27 2022
+++ sys/dev/wscons/wsconsio.h	Fri Jan 27 13:57:35 2023
@@ -662,4 +662,27 @@
 };
 #define WSMUXIO_LIST_DEVICES	_IOWR('W', 99, struct wsmux_device_list)
 
+/*
+ * Screendump ioctls (160 - 161)
+ */
+
+struct wsdisplay_screendump_text {
+	int rows;
+	int cols;
+	unsigned char * textdata;
+	unsigned int * attrdata;
+	};
+
+#define WSDISPLAYIO_SCREENDUMP_TEXT _IOWR('W', 160, struct wsdisplay_screendump_text)
+
+struct wsdisplay_screendump_pixels {
+	int width;
+	int height;
+	int stride;
+	int depth;
+	unsigned char * data;
+	};
+
+#define WSDISPLAYIO_SCREENDUMP_PIXELS _IOWR('W', 161, struct wsdisplay_screendump_pixels)
+
 #endif /* _DEV_WSCONS_WSCONSIO_H_ */
--- sys/dev/wscons/wsdisplayvar.h	Sun Sep 13 07:05:46 2020
+++ sys/dev/wscons/wsdisplayvar.h	Thu Jan 26 11:42:17 2023
@@ -84,6 +84,7 @@
 		    uint32_t *attrp);
 	void	(*unpack_attr)(void *c, uint32_t attr, int *fg, int *bg,
 		    int *ul);
+	int	(*screendump)(void *c, void *data);
 /* fg / bg values. Made identical to ANSI terminal color codes. */
 #define WSCOL_BLACK	0
 #define WSCOL_RED	1
--- sys/dev/wscons/wsdisplay.c	Fri Jul 15 14:57:27 2022
+++ sys/dev/wscons/wsdisplay.c	Fri Jan 27 13:59:33 2023
@@ -1148,6 +1148,39 @@
 	}
 
 	switch (cmd) {
+	case WSDISPLAYIO_SCREENDUMP_PIXELS:
+		{
+		int result;
+		struct wsemul_abortstate abortstate;
+		abortstate.skip=0;
+		WSEMULOP(result, scr->scr_dconf, &abortstate, screendump, (scr->scr_dconf->emulcookie, ((struct wsdisplay_screendump_pixels *)data)));
+		return (result);
+		}
+	case WSDISPLAYIO_SCREENDUMP_TEXT:
+		{
+		struct wsdisplay_charcell cell;
+#define SCREENDUMP ((struct wsdisplay_screendump_text *)data)
+#define DISP_SIZE (SCREENDUMP->rows * SCREENDUMP->cols)
+		unsigned char * text_data;
+		unsigned int * attr_data;
+		int i;
+		/* The values of the following assignments will be copied back to userspace upon exit. */
+		SCREENDUMP->rows=N_ROWS(scr->scr_dconf);
+		SCREENDUMP->cols=N_COLS(scr->scr_dconf);
+		text_data=malloc(DISP_SIZE, M_IOCTLOPS, M_WAITOK);
+		attr_data=malloc(DISP_SIZE*sizeof(unsigned int), M_IOCTLOPS, M_WAITOK);
+		for (i=0; i<DISP_SIZE; i++) {
+			GETCHAR(scr, i, &cell);
+			*(text_data + i)=cell.uc;
+			*(attr_data + i)=cell.attr;
+			}
+		/* Only need to use copyout for things passed as pointers */
+		copyout (text_data, SCREENDUMP->textdata, DISP_SIZE);
+		copyout (attr_data, SCREENDUMP->attrdata, DISP_SIZE*sizeof(unsigned int));
+		free (text_data, M_IOCTLOPS, DISP_SIZE);
+		free (attr_data, M_IOCTLOPS, DISP_SIZE*sizeof(unsigned int));
+		return (0);
+		}
 	case WSDISPLAYIO_GMODE:
 		if (scr->scr_flags & SCR_GRAPHICS) {
 			if (scr->scr_flags & SCR_DUMBFB)
--- sys/dev/rasops/rasops.h	Mon May 25 06:55:49 2020
+++ sys/dev/rasops/rasops.h	Thu Jan 26 11:40:25 2023
@@ -136,6 +136,7 @@
 	int	(*ri_copyrows)(void *, int, int, int);
 	int	(*ri_eraserows)(void *, int, int, uint32_t);
 	int	(*ri_pack_attr)(void *, int, int, int, uint32_t *);
+	int	(*ri_screendump)(void *, void *);
 };
 
 #define DELTA(p, d, cast) ((p) = (cast)((caddr_t)(p) + (d)))
--- sys/dev/rasops/rasops.c	Thu Jul 23 06:17:03 2020
+++ sys/dev/rasops/rasops.c	Thu Jan 26 11:41:14 2023
@@ -167,6 +167,7 @@
 int	rasops_eraserows_rotated(void *, int, int, uint32_t);
 int	rasops_putchar_rotated(void *, int, int, u_int, uint32_t);
 void	rasops_rotate_font(int *, int);
+int	rasops_screendump(void *, void *);
 
 /*
  * List of all rotated fonts
@@ -189,6 +190,7 @@
 int	rasops_vcons_eraserows(void *, int, int, uint32_t);
 int	rasops_vcons_pack_attr(void *, int, int, int, uint32_t *);
 void	rasops_vcons_unpack_attr(void *, uint32_t, int *, int *, int *);
+int	rasops_vcons_screendump(void *, void *);
 
 int	rasops_wronly_putchar(void *, int, int, u_int, uint32_t);
 int	rasops_wronly_copycols(void *, int, int, int, int);
@@ -284,6 +286,7 @@
 	ri->ri_copyrows = ri->ri_ops.copyrows;
 	ri->ri_eraserows = ri->ri_ops.eraserows;
 	ri->ri_pack_attr = ri->ri_ops.pack_attr;
+	ri->ri_screendump = ri->ri_ops.screendump;
 
 	if (ri->ri_flg & RI_VCONS) {
 		void *cookie;
@@ -307,6 +310,7 @@
 		ri->ri_ops.pack_attr = rasops_vcons_pack_attr;
 		ri->ri_ops.unpack_attr = rasops_vcons_unpack_attr;
 		ri->ri_do_cursor = rasops_wronly_do_cursor;
+		ri->ri_ops.screendump = rasops_vcons_screendump;
 	} else if ((ri->ri_flg & RI_WRONLY) && ri->ri_bs != NULL) {
 		uint32_t attr;
 		int i;
@@ -444,6 +448,7 @@
 	ri->ri_ops.unpack_attr = rasops_unpack_attr;
 	ri->ri_do_cursor = rasops_do_cursor;
 	ri->ri_updatecursor = NULL;
+	ri->ri_ops.screendump = rasops_screendump;
 
 	if (ri->ri_depth < 8 || (ri->ri_flg & RI_FORCEMONO) != 0) {
 		ri->ri_ops.pack_attr = rasops_pack_mattr;
@@ -2017,3 +2022,37 @@
 
 	return 0;
 }
+
+/*
+ * Screendump
+ */
+int
+rasops_screendump(void * cookie, void * data)
+{
+#define screendump ((struct wsdisplay_screendump_pixels *)data)
+#define FB_SIZE (ri->ri_stride * ri->ri_height)
+
+	struct rasops_info * ri;
+	unsigned char * rp;
+
+	ri=(struct rasops_info *) cookie;
+	rp=(unsigned char *)(ri->ri_bits);
+
+	/* Fill in various display parameters to pass back to wsdisplay, and eventually to userland. */
+	screendump->width=ri->ri_width;
+	screendump->height=ri->ri_height;
+	screendump->stride=ri->ri_stride;
+	screendump->depth=ri->ri_depth;
+
+	/* Copy the raw framebuffer data to userland. */
+	copyout (rp, screendump->data, FB_SIZE);
+	return (0);
+}
+
+int
+rasops_vcons_screendump(void * cookie, void * data)
+{
+	struct rasops_screen *scr = cookie;
+	return scr->rs_ri->ri_screendump(scr->rs_ri, data);
+}
+
--- sys/dev/ic/vga.c	Thu May 27 20:24:40 2021
+++ sys/dev/ic/vga.c	Fri Jan 27 14:10:47 2023
@@ -116,6 +116,7 @@
 int	vga_pack_attr(void *, int, int, int, uint32_t *);
 int	vga_copyrows(void *, int, int, int);
 void	vga_unpack_attr(void *, uint32_t, int *, int *, int *);
+int	vga_screendump(void *, void *);
 
 static const struct wsdisplay_emulops vga_emulops = {
 	pcdisplay_cursor,
@@ -126,7 +127,8 @@
 	vga_copyrows,
 	pcdisplay_eraserows,
 	vga_pack_attr,
-	vga_unpack_attr
+	vga_unpack_attr,
+	vga_screendump
 };
 
 /*
@@ -1245,6 +1247,12 @@
 
 		vga_loadchars(&vc->hdl, slot, 0, 256, f->height, f->fontdata);
 	}
+}
+
+int
+vga_screendump(void * cookie, void * unused)
+{
+return (EINVAL);
 }
 
 #ifdef __alpha__
