/*
 * Copyright 2023, Exotic Silicon, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. This software is licensed exclusively under this specific license text.  The license text may not be changed, and the software
 *    including modified versions may not be re-licensed under any other license text.
 * 2. Redistributions of source code must retain the above copyright notice, this list of conditions, and the following disclaimer.
 * 3. Redistributions in binary form must reproduce the above copyright notice, this list of conditions, and the following
 *    disclaimer in the documentation and/or other materials provided with the distribution.
 * 4. All advertising materials mentioning features or use of this software must display the following acknowledgement: This product
 *    includes software developed by Exotic Silicon.
 * 5. The name of Exotic Silicon must not be used to endorse or promote products derived from this software without specific prior
 *    written permission.
 * 6. Redistributions of modified versions of the source code must be clearly identified as having been modified from the original.
 * 7. Redistributions in binary form that have been created from modified versions of the source code must clearly state in the
 *    documentation and/or other materials provided with the distribution that the source code has been modified from the original.
 *
 * THIS SOFTWARE IS PROVIDED 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EXOTIC SILICON BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Graphical screendump utility, version 1.0.
 *
 * For more information about this utility, and the kernel patches it requires, please visit:
 * https://research.exoticsilicon.com/articles/console_screendumps
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <dev/wscons/wsconsio.h>
#include <fcntl.h>

int main()
{
int fd_out;
int header_len;
int pos_in;
int pos_out;
int res;
int x;
int y;
struct wsdisplay_screendump_pixels screendump;
struct wsdisplay_fbinfo dispinfo;
unsigned char * header;
unsigned char * file_out;
unsigned char * processed_pixel_data;

/* Call WSDISPLAYIO_GINFO to get the size of the framebuffer. */
res=ioctl (STDIN_FILENO, WSDISPLAYIO_GINFO, &dispinfo);

if (res == -1) {
	printf ("Ioctl WSDISPLAYIO_GINFO not supported for this device\n");
	return (1);
	}

/* Allocate the correct amount of memory for the screendump. */
screendump.data=malloc(dispinfo.stride * dispinfo.height);

res=ioctl (STDIN_FILENO, WSDISPLAYIO_SCREENDUMP_PIXELS, &screendump);

if (res == -1) {
	printf ("Ioctl WSDISPLAYIO_SCREENDUMP_PIXELS not supported for this device\n");
	return (1);
	}

printf ("Ioctl WSDISPLAYIO_GINFO             returned %d\nwidth %d, height %d, stride %d, depth %d\n", res, dispinfo.width, dispinfo.height, dispinfo.stride, dispinfo.depth);
printf ("Ioctl WSDISPLAYIO_SCREENDUMP_PIXELS returned %d\nwidth %d, height %d, stride %d, depth %d\n", res, screendump.width, screendump.height, screendump.stride, screendump.depth);

if (screendump.depth != 32) {
	printf ("Only 32bpp input data is supported.\n");
	return (1);
	}

/*
 * Simple conversion to 24-bit RGB
 *
 * This assumes that the raw screendump data is in BGR0 format.
 */

processed_pixel_data=malloc(screendump.width * screendump.height * 4);

pos_out=0;
pos_in=0;

for (y=0; y<screendump.height; y++) {
	for (x=0; x<screendump.stride/4; x++) {
		if (x<(screendump.width)) {
			*(processed_pixel_data+pos_out++)=*(screendump.data+pos_in+2);
			*(processed_pixel_data+pos_out++)=*(screendump.data+pos_in+1);
			*(processed_pixel_data+pos_out++)=*(screendump.data+pos_in);
		}
		pos_in+=4;
		}
	}

header=malloc(1024);
header_len=sprintf (header, "P6\n%d %d\n255\n", screendump.width, screendump.height);

fd_out=open ("rgb_screendump.ppm", O_CREAT | O_WRONLY, 0644);
write (fd_out, header, header_len);
write (fd_out, processed_pixel_data, pos_out);
close (fd_out);
return (0);
}
